﻿angular.module("umbraco.directives")
    .directive('flowDeploy', function (deployService, taskManService) {
        return {
            transclude: true,
            restrict: 'E',
            replace: true,
            templateUrl: '/app_plugins/deploy/views/flow-deploy.html',
            scope: {
                currentTask: "=?",
                environment: "=",
                user: "="     
            },
            link: function (scope, element, attrs) {

                var deploying = false;
                var subscribed = false;
                //used to track the tasks that we have started, in some cases a task would be started
                // from a different dashboard or dialog, we don't want to perform actions based on the results
                // from those polls. We may want to update our UI based on them but not perform actions.
                var managedTaskId = null;

                //Handles errors and shows in the UI
                function handleError(err) {

                    deploying = false;

                    if (err.status === 500) {
                        //this is a ysod/unhandled exception
                        scope.error = {
                            message: err.data.Message,
                            exceptionMessage: err.data.ExceptionMessage,
                            stackTrace: err.data.StackTrace
                        }
                        scope.deploystep = "exception";
                    }
                    else {
                        scope.deploystep = "errors";
                        scope.error = err.data ? err.data : err;
                    }
                }

                //clear the collected changes to start over
                function clearQueue() {
                    return deployService.clearQueue().then(function () {
                        scope.deploystep = "";
                        scope.manifest = undefined;
                        scope.editorChanges = 0;
                    }, handleError);
                }

                function reloadQueue()
                {
                    //get the current manifest the editor is adding items to
                    deployService.getManifest().then(function (response) {
                        scope.manifest = response.data;
                        scope.editorChanges = 0;
                        _.each(scope.manifest.Providers, function (provider) {
                            if (provider.IncludeAll === true) {
                                scope.editorChanges++;
                            }
                            scope.editorChanges += provider.Items.length;
                            provider.name = deployService.itemProviderName(provider.Id).name;
                        });

                        if (response.data.Providers.length === 0) {
                            clearQueue();
                        } else {
                            scope.deploystep = "current-queue";
                        }
                    }, handleError);
                }

                function init() {

                    //on init, check if a task is running already
                    if (taskManService.currentTask && !taskManService.currentTask.complete) {
                        scope.deploystep = "working";
                    }
                    else {
                        reloadQueue();
                    }
                    
                    taskManService.subscribe();
                    subscribed = true;
                }

                //handle task manager updates
                var onTaskUpDate = function (event, task) {
                    scope.currentTask = task;
                    if (scope.deploystep === "empty-queue" || scope.deploystep === "done") {
                        scope.deploystep = "working";
                        deploying = true;
                    }
                };
                //handle task manager complete
                var onTaskComplete = function (event, task) {
                    scope.currentTask = task;

                    //when a task completes, the UI might do different things...

                    //we are done packaging and now needs to show the user the packaged items
                    // only proceed to execute this if we are not currently in the deploying process and the task Id matches one
                    // that we've actually started.
                    if (task.name === "Umbraco.Courier.Core.Tasks.PackagingTask" && deploying === false && scope.currentTask.id === managedTaskId) {
                        scope.showPackagedItems();
                        deploying = true;
                    }

                    //we are done extracting / deploying to a target, we will just show the done page
                    if (task.name === "Umbraco.Courier.Core.Tasks.ExtractionTask") {
                        scope.deploystep = "done";
                        deploying = false;
                    }

                    //we just completed a restore, should we show anything? - we cant tell the difference between
                    //nonodes restore and dialog restore since its the same task
                    if (task.name === "Concorde.CacheHandler.Tasks.InitialExtractionTask") {
                        scope.deploystep = "restored";
                        deploying = false;
                    }
                };

                //when a change is added
                deployService.events.on("add", reloadQueue);

                //when a task updates
                taskManService.events.on("task-update", onTaskUpDate);

                //when a task completes, move to done, or move to showing what has been packaged
                taskManService.events.on("task-complete", onTaskComplete);

                //when an error occurs in polling - this will be a critical error
                taskManService.events.on("error", function (evt, msg) {
                    handleError(msg);

                    //there's an error, we cannot continue or we'll overwrite the UI message that the user must see
                    // if an error occurs, this is critical, we cannot swallow/hide issues.
                    taskManService.unsubscribe();
                    subscribed = false;
                });

                scope.error = {};
                scope.deploystep = "";
                if (!scope.editorChanges) {
                    scope.editorChanges = 0;
                }

                scope.clear = clearQueue;
               
                //deploy the packaged revision to the remote host
                scope.deploy = function() {
                    scope.deploystep = "deploy";

                    deployService.deployQueue().then(function (response) {
                        //we get a task back
                        scope.currentTask = response.data;
                    }, handleError);
                }

                //reload the queue
                scope.reloadQueue = reloadQueue;

                //package the collected items in the manifest
                scope.package = function () {

                    deployService.package().then(function (response) {
                        scope.deploystep = "pre-collect";

                        //store the task we've started
                        managedTaskId = response.data.id;
                        scope.currentTask = response.data;

                        if (!subscribed) {
                            taskManService.subscribe();
                            subscribed = true;
                        }

                    }, handleError);
                };

                //show the items packaged by courier
                scope.showPackagedItems = function () {
                    //if the task is completed
                    //load the deployment
                    deployService.getDeployment().then(function (response) {

                        //iterate through all the returned data for better view
                        var notallowed = [];
                        angular.forEach(response.data.notAllowed, function (value, key) {
                            var provider = deployService.itemProviderName(key);
                            provider.items = value;
                            notallowed.push(provider);
                            response.data.notAllowed = notallowed;
                        });

                        scope.manifest = response.data;

                        if (Object.keys(scope.manifest.notAllowed).length > 0) {
                            scope.deploystep = "problems";
                            taskManService.unsubscribe();
                            subscribed = false;
                            deploying = false;
                        }
                        else {
                            if (Object.keys(scope.manifest.deployment).length === 0) {
                                deployService.clearQueue();
                                scope.deploystep = "empty";
                            }
                            else {
                                scope.deploystep = "collect";
                            }
                        }
                    }, handleError);
                };

                scope.$on(
                    "$destroy",
                    function (event) {
                        deployService.events.off("add", reloadQueue);
                        taskManService.events.off("task-update", onTaskUpDate);
                        taskManService.events.off("task-complete", onTaskComplete);
                        taskManService.unsubscribe();
                        subscribed = false;
                    }
                );

                init();

            }
        };
    });